# -*- coding: utf-8 -*-
"""
Authors: Peter Mawhorter
Consulted:
Date: 2022-4-5
Purpose: predictText task tests.
"""

import optimism

import predictText

import random, time

#-------#
# Setup #
#-------#


def buildString(wordsList):
    """
    Turns a list-of-words into a string to be printed. Inserts spaces
    between each pair of words, and capitalizes words when the previous
    word ends with '.', '?', or '!' (ignoring quotation marks after such
    punctuation).
    """
    if wordsList is None:
        return "(None)"
    result = ""
    capitalize = True
    for word in wordsList:
        if capitalize or word == 'i' or word.startswith("i'"):
            result += ' ' + word.capitalize()
        else:
            result += ' ' + word

        # Determine capitalization of next word
        if word.rstrip('\'"’”)')[-1:] in ('.', '?', '!'):
            capitalize = True
        else:
            capitalize = False

    return result[1:] # chop off the extra space at the beginning


blueWhale = [ "The", "blue", "whale", "wore", "the", "blue", "hat." ]
cagedBird = [ # by Maya Angelou
     "A", "free", "bird", "leaps",
     "on", "the", "back", "of", "the", "wind",
     "and", "floats", "downstream",
     "till", "the", "current", "ends",
     "and", "dips", "his", "wing",
     "in", "the", "orange", "sun", "rays",
     "and", "dares", "to", "claim", "the", "sky.",
     "But", "a", "bird", "that", "stalks",
     "down", "his", "narrow", "cage",
     "can", "seldom", "see", "through",
     "his", "bars", "of", "rage",
     "his", "wings", "are", "clipped", "and",
     "his", "feet", "are", "tied",
     "so", "he", "opens", "his", "throat", "to", "sing.",
     "The", "caged", "bird", "sings",
     "with", "a", "fearful", "trill",
     "of", "things", "unknown",
     "but", "longed", "for", "still",
     "and", "his", "tune", "is", "heard",
     "on", "the", "distant", "hill",
     "for", "the", "caged", "bird",
     "sings", "of", "freedom.",
     "The", "free", "bird", "thinks", "of", "another", "breeze",
     "and", "the", "trade", "winds", "soft", "through", "the", "sighing",
            "trees",
     "and", "the", "fat", "worms", "waiting", "on", "a", "dawn", "bright",
            "lawn",
     "and", "he", "names", "the", "sky", "his", "own",
     "But", "a", "caged", "bird", "stands", "on", "the", "grave", "of",
            "dreams",
     "his", "shadow", "shouts", "on", "a", "nightmare", "scream",
     "his", "wings", "are", "clipped", "and", "his", "feet", "are", "tied",
     "so", "he", "opens", "his", "throat", "to", "sing.",
     "The", "caged", "bird", "sings",
     "with", "a", "fearful", "trill",
     "of", "things", "unknown",
     "but", "longed", "for", "still",
     "and", "his", "tune", "is", "heard",
     "on", "the", "distant", "hill",
     "for", "the", "caged", "bird",
     "sings", "of", "freedom.",
]


#---------#
# Testing #
#---------#


# To show full results, turn up the detail level by uncommenting this
# optimism.detailLevel(1)

testBP = optimism.testFunctionMaybe(predictText, "buildPredictions")
testBP.case(blueWhale).checkReturnValue({
    "the": ["blue", "blue"],
    "blue": ["whale", "hat."],
    "whale": ["wore"],
    "wore": ["the"]
})
testBP.case(cagedBird).checkReturnValue({
    'a': ['free', 'bird', 'fearful', 'dawn', 'caged', 'nightmare',
          'fearful'],
    'free': ['bird', 'bird'],
    'bird': ['leaps', 'that', 'sings', 'sings', 'thinks', 'stands',
             'sings', 'sings'],
    'leaps': ['on'],
    'on': ['the', 'the', 'a', 'the', 'a', 'the'],
    'the': ['back', 'wind', 'current', 'orange', 'sky.', 'caged',
            'distant', 'caged', 'free', 'trade', 'sighing', 'fat',
            'sky', 'grave', 'caged', 'distant', 'caged'],
    'back': ['of'],
    'of': ['the', 'rage', 'things', 'freedom.', 'another', 'dreams',
           'things', 'freedom.'],
    'wind': ['and'],
    'and': ['floats', 'dips', 'dares', 'his', 'his', 'the', 'the',
            'he', 'his', 'his'],
    'floats': ['downstream'],
    'downstream': ['till'],
    'till': ['the'],
    'current': ['ends'],
    'ends': ['and'],
    'dips': ['his'],
    'his': ['wing', 'narrow', 'bars', 'wings', 'feet', 'throat',
            'tune', 'own', 'shadow', 'wings', 'feet', 'throat',
            'tune'],
    'wing': ['in'],
    'in': ['the'],
    'orange': ['sun'],
    'sun': ['rays'],
    'rays': ['and'],
    'dares': ['to'],
    'to': ['claim', 'sing.', 'sing.'],
    'claim': ['the'],
    'sky.': ['but'],
    'but': ['a', 'longed', 'a', 'longed'],
    'that': ['stalks'],
    'stalks': ['down'],
    'down': ['his'],
    'narrow': ['cage'],
    'cage': ['can'],
    'can': ['seldom'],
    'seldom': ['see'],
    'see': ['through'],
    'through': ['his', 'the'],
    'bars': ['of'],
    'rage': ['his'],
    'wings': ['are', 'are'],
    'are': ['clipped', 'tied', 'clipped', 'tied'],
    'clipped': ['and', 'and'],
    'feet': ['are', 'are'],
    'tied': ['so', 'so'],
    'so': ['he', 'he'],
    'he': ['opens', 'names', 'opens'],
    'opens': ['his', 'his'],
    'throat': ['to', 'to'],
    'sing.': ['the', 'the'],
    'caged': ['bird', 'bird', 'bird', 'bird', 'bird'],
    'sings': ['with', 'of', 'with', 'of'],
    'with': ['a', 'a'],
    'fearful': ['trill', 'trill'],
    'trill': ['of', 'of'],
    'things': ['unknown', 'unknown'],
    'unknown': ['but', 'but'],
    'longed': ['for', 'for'],
    'for': ['still', 'the', 'still', 'the'],
    'still': ['and', 'and'],
    'tune': ['is', 'is'],
    'is': ['heard', 'heard'],
    'heard': ['on', 'on'],
    'distant': ['hill', 'hill'],
    'hill': ['for', 'for'],
    'freedom.': ['the'],
    'thinks': ['of'],
    'another': ['breeze'],
    'breeze': ['and'],
    'trade': ['winds'],
    'winds': ['soft'],
    'soft': ['through'],
    'sighing': ['trees'],
    'trees': ['and'],
    'fat': ['worms'],
    'worms': ['waiting'],
    'waiting': ['on'],
    'dawn': ['bright'],
    'bright': ['lawn'],
    'lawn': ['and'],
    'names': ['the'],
    'sky': ['his'],
    'own': ['but'],
    'stands': ['on'],
    'grave': ['of'],
    'dreams': ['his'],
    'shadow': ['shouts'],
    'shouts': ['on'],
    'nightmare': ['scream'],
    'scream': ['his']
})

if not hasattr(predictText, "buildPredictions"):
    print(
        "Stopping tests early because `buildPredictions` is not yet"
        " defined."
    )
else:
    bluePred = predictText.buildPredictions(blueWhale)
    birdPred = predictText.buildPredictions(cagedBird)

    testNW = optimism.testFunctionMaybe(predictText, "nextWord")
    # Set a specific random seed so we can test the random policy
    # predictably
    random.seed(19281912)
    case1 = testNW.case("key", {'key': ['value']}, "default shouldn't be used")
    case1.checkReturnValue('value')
    case2 = testNW.case("the", bluePred, "default shouldn't be used")
    case2.checkReturnValue('blue')
    case3 = testNW.case("blue", bluePred, "default shouldn't be used")
    case3.checkReturnValue('hat.')
    case4 = testNW.case("missing", bluePred, "default")
    case4.checkReturnValue('default')
    case5 = testNW.case("the", birdPred, "default shouldn't be used")
    case5.checkReturnValue('orange')

    # Reset to a specific random seed so we can test the random policy
    # predictably
    random.seed(19281912)

    testPT = optimism.testFunctionMaybe(predictText, "predictText")
    print("""\
    Note: the next two random tests will only work correctly if you use
    the provided `chooseRandomly` function, and only if you use it
    exactly once per word generated.""")
    testPT.case("the", 15, bluePred).checkReturnValue([
        'the', 'blue', 'whale', 'wore', 'the', 'blue', 'hat.', 'the',
        'blue', 'whale', 'wore', 'the', 'blue', 'hat.', 'the', 'blue'
    ])
    testPT.case("of", 15, birdPred).checkReturnValue([
        'of', 'things', 'unknown', 'but', 'longed', 'for', 'still', 'and',
        'his', 'shadow', 'shouts', 'on', 'the', 'distant', 'hill', 'for'
    ])

    # Here at the end, we use the random policy a few times with each
    # predictions dictionary and just print the results
    print("""\
    The next few lines are randomly generated; you can inspect them to
    make sure that words are always following other words that appeared
    before them in the actual source material.""")

    # Pick a 'random' seed based on the current time
    random.seed(time.monotonic())
    print('---')
    print(buildString(predictText.predictText("the", 10, bluePred)))
    print(buildString(predictText.predictText("the", 10, bluePred)))
    print(buildString(predictText.predictText("the", 10, bluePred)))
    print('---')
    print(buildString(predictText.predictText("the", 10, birdPred)))
    print(buildString(predictText.predictText("the", 10, birdPred)))
    print(buildString(predictText.predictText("the", 10, birdPred)))
